package client

import (
	"fmt"
	"net/http"
	"random.chars.jp/git/image-board/api"
	"random.chars.jp/git/image-board/store"
)

// Remote represents a remote image board server.
type Remote struct {
	url     string
	single  bool
	private bool
	secret  string
	client  *http.Client
	user    *api.UserPayload
	store.Info
}

// New returns a pointer to a new Remote.
func New(url string) (*Remote, error) {
	remote := &Remote{url: url, client: &http.Client{}}
	return remote, remote.Handshake()
}

// URL returns the URL of Remote.
func (r *Remote) URL(path string) string {
	return r.url + path
}

// SingleUser returns whether the Remote is running in single user mode.
func (r *Remote) SingleUser() bool {
	return r.single
}

// Private returns whether the Remote is running in private mode.
func (r *Remote) Private() bool {
	return r.private
}

// Handshake checks if the server is still online and updates Remote.
func (r *Remote) Handshake() error {
	if err := r.fetch(http.MethodGet, api.SingleUser, &r.single, nil); err != nil {
		return err
	}
	if err := r.fetch(http.MethodGet, api.Private, &r.private, nil); err != nil {
		return err
	}
	return r.fetch(http.MethodGet, api.Base, r, nil)
}

// Secret authenticates and sets secret.
func (r *Remote) Secret(secret string) (api.UserPayload, bool) {
	prev := r.secret
	r.secret = secret
	if user, err := r.This(); err != nil {
		fmt.Printf("Error getting user post secret change, %s", err)
		r.secret = prev
		return api.UserPayload{}, false
	} else {
		r.user = &user
		return user, true
	}
}
